#ifndef _QIC80_VTBL_H
#define _QIC80_VTBL_H
/*
 *
 *      Copyright (c) 1995  Claus-Justus Heine (cH)
 *
 *      This file defines a volume table as defined in the QIC-80 development 
 *      standards defined.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * $Author: claus $
 *
 * $Revision: 0.1 $
 * $Date: 1995/8/3 16:07:53 $
 * $State: Beta $
 *
 */

#include "ftape.h"
#include "ftape-eof.h"

struct qic80_vtbl 
{
  unsigned long signature;            /* must be "VTBL" */
  unsigned short first_segment; /* first segment of this volume (2 bytes) */
  unsigned short last_segment;  /* last segment of this volume (2 bytes)  */
  char description[44];         /* any string */
  unsigned long date;           /* encoded date */
  union {
    struct {
      byte vendorspecific     : 1; /* rem. of vtbl entry is vendor specific */
      byte multicartridge     : 1; /* self explanatory */
      byte not_verified       : 1; /* set if file set wasn't verified */
      byte redirection_inhibit: 1; /* file set redirection is not allowed??? */
      byte segment_spanning   : 1; /* til QIC-80 Rev. H: always 0
                                    * Rev. I til ...   : 0 if segment spanning 
                                    *                      wasn't used
                                    *                    1 if segment spanning 
                                    *                       was used
                                    */
      byte unused5 : 1;
      byte unused6 : 1;
      byte unused7 : 1;
    } flags;
    byte dummy;
  } flags;
  byte multi_cartridge_count;
  byte vendor_extension[26];

  char password[8];
  unsigned long dir_section_size; /* size of directory entry at begin of 
                                   * fileset 
                                   */
  unsigned long data_section_size; /* size of data section, including 
                                    * file-headers 
                                    */
  byte OS_version_major;
  byte OS_version_minor;
  char source_volume[16];
  byte logical_dev_file_set;
  byte physical_dev_file_set;
  byte QIC_compression_method;
  union {
    struct {
      byte OS_DOS   : 1;
      byte OS_UNIX  : 1;
      byte OS_OS_2  : 1;
      byte OS_MAC   : 1;
      byte OS_NOVELL: 1;
      byte OS_LAN   : 1;
      byte OS_unused6 : 1;
      byte OS_unused7 : 1;
    } flags;
    byte dummy;
  } OS_flags;
  byte OS_flag_dummy;
  byte ISO_compression_method;
  byte reserved[4];
};

/*
 *  one vtbl is 128 bytes, that results in a maximum number of
 *  29*1024/128 = 232 volumes. 
 */
#define MAX_VOLUMES (29*SECTOR_SIZE/sizeof(struct qic80_vtbl) )
#define VTBL_ID "VTBL"
#define VOL_NAME "zftape volume" /* volume label used by me */

/*
 *  global variables
 */
extern int ftape_qic80_mode;
extern int ftape_old_ftape;
extern int ftape_volume_table_changed;

/*
 *  exported functions
 */
extern void qic80_init_vtbl                ( void );
extern void qic80_create_volume_headers    ( byte *buffer );
extern void qic80_extract_volume_headers   ( byte *buffer );
extern int  qic80_update_volume_table      ( unsigned segment, byte *buffer );
extern int  qic80_maybe_write_beof         ( unsigned segment,
                                             unsigned no_new_one );
extern void qic80_write_eof                ( unsigned segment, 
                                             unsigned long data_pos );
extern int  qic80_check_for_eof            ( unsigned segment, 
                                             unsigned long data_pos, 
                                             long *distance );
extern int  qic80_skip_files               ( unsigned segment, int count );
extern int  qic80_check_for_eom            ( unsigned segment );
extern void qic80_skip_to_eom              ( void );
extern int  qic80_find_file_no             ( unsigned segment );
extern int  qic80_get_volume_block_size    ( unsigned segment );
extern int  qic80_get_volume_size          ( unsigned segment );
extern int  qic80_get_volume_cmpr_flag     ( unsigned segment );
extern int  qic80_get_volume_boundaries    ( unsigned segment,
                                             unsigned *end_seg );
extern void qic80_fake_volume_headers      ( eof_mark_union *eof_map, 
                                             int num_failed_sectgors );
extern void qic80_fake_single_volume_header( unsigned start_seg, 
                                             unsigned end_seg, 
                                             unsigned long size );
extern int  qic80_weof                     ( unsigned count );



#endif
/*{{{}}}*/
